package com.chelvc.framework.security.interceptor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import javax.crypto.Cipher;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.base.interceptor.BufferedRequestWrapper;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.security.annotation.Crypto;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdviceAdapter;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

/**
 * 接口参数加解密拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@ControllerAdvice
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class ControllerCryptoInterceptor extends RequestBodyAdviceAdapter implements ResponseBodyAdvice<Object> {
    private final SecurityProperties properties;

    /**
     * 获取密码处理器
     *
     * @param mode 加解密模式
     * @return 密码处理器实例
     */
    private Cipher getCipher(int mode) {
        String secret = AssertUtils.nonempty(this.properties.getSecret(), () -> "Secret is missing");
        return AESUtils.lookupDefaultCipher(mode, secret);
    }

    /**
     * 获取目标方法@Crypto注解
     *
     * @param method 目标方法
     * @return @Crypto注解实例
     */
    private Crypto getCryptoAnnotation(MethodParameter method) {
        Crypto annotation = method.getMethodAnnotation(Crypto.class);
        if (annotation == null) {
            annotation = method.getDeclaringClass().getAnnotation(Crypto.class);
        }
        return annotation;
    }

    /**
     * 获取消息密文
     *
     * @param message 消息对象
     * @return 密文字节数组
     * @throws IOException I/O异常
     */
    private byte[] getCiphertext(HttpInputMessage message) throws IOException {
        InputStream input = message.getBody();
        if (input instanceof BufferedRequestWrapper.BufferedServletInputStream) {
            return ((BufferedRequestWrapper.BufferedServletInputStream) input).decodeBase64();
        }
        return Base64.decodeBase64(FileUtils.getBytes(input));
    }

    @Override
    public boolean supports(MethodParameter method, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.output();
    }

    @Override
    public boolean supports(MethodParameter method, Type type, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.input();
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage message, MethodParameter method, Type type,
                                           Class<? extends HttpMessageConverter<?>> clazz) throws IOException {
        Cipher cipher = this.getCipher(Cipher.DECRYPT_MODE);
        byte[] plaintext, ciphertext = this.getCiphertext(message);
        try {
            plaintext = AESUtils.codec(cipher, ciphertext);
        } catch (Exception e) {
            log.warn("Request parameter decrypt failed: {}", e.getMessage());
            throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                    ApplicationContextHolder.getMessage("Request.Invalid"));
        }
        return new HttpInputMessage() {
            @Override
            public HttpHeaders getHeaders() {
                return message.getHeaders();
            }

            @Override
            public InputStream getBody() throws IOException {
                return new ByteArrayInputStream(plaintext);
            }
        };
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter method, MediaType media,
                                  Class<? extends HttpMessageConverter<?>> clazz,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        if (body == null) {
            return null;
        }
        if (body instanceof Result) {
            Result<?> result = (Result<?>) body;
            if (result.getData() == null || result.isFailure()) {
                return result;
            }
            Cipher cipher = this.getCipher(Cipher.ENCRYPT_MODE);
            String ciphertext = AESUtils.encrypt(cipher, JacksonContextHolder.serialize(result.getData()));
            return Result.of(result.getCode(), ciphertext, result.getMessage());
        }
        Cipher cipher = this.getCipher(Cipher.ENCRYPT_MODE);
        return AESUtils.encrypt(cipher, JacksonContextHolder.serialize(body));
    }
}
